package.loaded.Constants = nil; require("Constants")

CrabBehaviors = {}

-- move to the next waypoint
function CrabBehaviors.GoToWpt(self)
	if not self.MOMoveTarget then
		if SceneMan:ShortestDistance(self:GetLastAIWaypoint(), self.Pos, false).Magnitude < 10 then
			self:ClearAIWaypoints()
			if self.AIMode == Actor.AIMODE_GOTO then
				self.AIMode = Actor.AIMODE_SENTRY
			end
			return true
		end
	end
	
	-- is Y1 lower down in the scene, compared to Y2?
	local Lower = function(Y1, Y2, margin)
		return Y1.Pos.Y - margin > Y2.Pos.Y
	end
	
	-- is Y1 higher up in the scene, compared to Y2?
	local Higher = function(Y1, Y2, margin)
		return Y1.Pos.Y + margin < Y2.Pos.Y
	end
	
	local AngleBetweenVectors = function(A, B)	-- returns the smallest angle between two vectors
		local angle = math.atan2(A.Y, A.X) - math.atan2(B.Y, B.X)
		
		if angle > 3.142 then
		 angle = angle - 6.283
		elseif angle < -3.142 then
		 angle = angle + 6.283
		end
		
		return angle
	end
	
	local UpdatePathTimer = Timer()
	local ArrivedTimer = Timer()
	local BurstTimer = Timer()
	
	local NoLOSTimer = Timer()
	NoLOSTimer:SetSimTimeLimitMS(1000)
	
	local StuckTimer = Timer()
	StuckTimer:SetSimTimeLimitMS(2000)
	
	local nextLatMove = self.lateralMoveState
	local nextAimAngle = self:GetAimAngle(false) * 0.8
	local scanAng = 0	-- for obstacle detection
	local Obstacles = {}
	local PrevWptPos = self.Pos
	local sweepCW = true
	local sweepRange = 0
	local digState = AHuman.NOTDIGGING
	local obstacleState = Actor.PROCEEDING
	local WptList, Waypoint, Dist, CurrDist
	
	while true do
		while not self.flying and self.Target do	-- don't move around if we have something to shoot at
			coroutine.yield()	-- wait until next frame
		end
		
		if self.Vel.Magnitude > 1 then
			StuckTimer:Reset()
		end
		
		if self.MOMoveTarget then	-- make the last waypoint marker stick to the MO we are following
			if MovableMan:ValidMO(self.MOMoveTarget) then
				self:RemoveMovePathEnd()
				self:AddToMovePathEnd(self.MOMoveTarget.Pos)
				
				if self.Team == self.MOMoveTarget.Team and not self.flying then	-- we are following an ally, stop when close
					while SceneMan:ShortestDistance(self.Pos, self.MOMoveTarget.Pos, false).Magnitude < self.Radius+self.MOMoveTarget.Radius+10 do
						coroutine.yield()	-- wait until next frame
						
						if not MovableMan:ValidMO(self.MOMoveTarget) then
							self:ClearMovePath()
							break
						end
					end
				end
			else
				self:ClearMovePath()
			end
		end
		
		if not self.flying and UpdatePathTimer:IsPastSimMS(6000) then
			if Waypoint and self.BlockingActor then
				if MovableMan:IsActor(self.BlockingActor) then
					CurrDist = SceneMan:ShortestDistance(self.Pos, Waypoint.Pos, false)
					if (self.Pos.X > self.BlockingActor.Pos.X and CurrDist.X < self.Pos.X) or
						(self.Pos.X < self.BlockingActor.Pos.X and CurrDist.X > self.Pos.X) or
						SceneMan:ShortestDistance(self.Pos, self.BlockingActor.Pos, false).Magnitude > self.Diameter + self.BlockingActor.Diameter
					then
						self.BlockingActor = nil	-- the blocking actor is not in the way any longer
						self.teamBlockState = Actor.NOTBLOCKED
					else
						self.BlockedTimer:Reset()
						self.teamBlockState = Actor.IGNORINGBLOCK
						self:CreateMoveAroundBehavior()
						break	-- end this behavior
					end
				else
					self.BlockingActor = nil
				end
			end
			
			UpdatePathTimer:Reset()
			self.deviceState = AHuman.STILL
			self.crawling = false
			self.jump = false
			nextLatMove = Actor.LAT_STILL
			Waypoint = nil
			WptList = nil
		elseif StuckTimer:IsPastSimTimeLimit() then	-- dislodge
			if self.jump then
					local chance = PosRand()
					if chance < 0.1 then
						nextLatMove = Actor.LAT_LEFT
					elseif chance > 0.9 then
						nextLatMove = Actor.LAT_RIGHT
					else
						nextLatMove = Actor.LAT_STILL
					end
				--end
			else
				if PosRand() < 0.2 then
					if self.lateralMoveState == Actor.LAT_LEFT then
						nextLatMove = Actor.LAT_RIGHT
					else
						nextLatMove = Actor.LAT_LEFT
					end
				end
				self.jump = true
				
				--if self.refuel and self.JetTimeLeft > self.JetTimeTotal * 0.95 then
					--[[
					BurstTimer:Reset()
					self.jump = true
					BurstTimer:SetSimTimeLimitMS(5000)	-- this burst last until the BurstTimer expire
					]]
				--end
			end
		elseif WptList then	-- we have a list of waypoints, folow it
			if #WptList < 1 and not Waypoint then	-- arrived
				if self.MOMoveTarget and MovableMan:ValidMO(self.MOMoveTarget) then
					if self.Team ~= self.MOMoveTarget.Team then
						Waypoint = {Pos=self.MOMoveTarget.Pos}
					else
						nextLatMove = Actor.LAT_STILL
						self.jump = false
						coroutine.yield()	-- wait until next frame
					end
				else
					break
				end
			else
				if not Waypoint then	-- get the next waypoint in the list					
					-- we must update the path more often if we are following someone
					if self.MOMoveTarget and MovableMan:ValidMO(self.MOMoveTarget) then
						if SceneMan:ShortestDistance(self:GetLastAIWaypoint(), self.MOMoveTarget.Pos, false).Magnitude > 100 then	-- the target has moved
							WptList = nil
							Waypoint = nil
						else
							UpdatePathTimer:Reset()
						end
					end
					
					if WptList then						
						local NextWptPos = WptList[1].Pos
						Dist = SceneMan:ShortestDistance(self.Pos, NextWptPos, false)
						if Dist.Y < -25 and math.abs(Dist.X) < 30 then	-- avoid any corners if the next waypoint is above us
							local CornerPos = Vector(NextWptPos.X, NextWptPos.Y)
							if self.Pos.X > CornerPos.X then
								CornerPos = CornerPos + Vector(25, -40)
								cornerType = "right"
							else
								CornerPos = CornerPos + Vector(-25, -40)
								cornerType = "left"
							end
							
							local Free = Vector()
							Dist = SceneMan:ShortestDistance(NextWptPos, CornerPos, false)
							-- make sure the corner waypoint is not inside terrain
							local pixels = SceneMan:CastObstacleRay(NextWptPos, Dist, Vector(), Free, self.ID, self.IgnoresWhichTeam, rte.grassID, 3)
							if pixels == 0 then
								break	-- the waypoint is inside terrain, plot a new path
							elseif pixels > 0 then
								CornerPos = (NextWptPos + Free) / 2	-- compensate for obstacles
							end
							
							coroutine.yield()	-- wait until next frame
							
							-- check if we have LOS
							Dist = SceneMan:ShortestDistance(self.Pos, CornerPos, false)
							if 0 <= SceneMan:CastObstacleRay(self.Pos, Dist, Vector(), Vector(), self.ID, self.IgnoresWhichTeam, rte.grassID, 4) then
								-- CornerPos is blocked
								CornerPos.X = self.Pos.X	-- move CornerPos straight above us
								cornerType = "air"
							end
							
							coroutine.yield()	-- wait until next frame
							
							Waypoint = {Pos=CornerPos, Type=cornerType}
							if #WptList > 1 and not WptList[1].Type then	-- remove the waypoint after the corner if possible
								table.remove(WptList, 1)
								self:RemoveMovePathBeginning() -- clean up the graphical representation of the path
							end
							
							self:AddToMovePathBeginning(Waypoint.Pos)
						else
							Waypoint = table.remove(WptList, 1)							
							if Waypoint.Type ~= "air" then
								local Free = Vector()
								
								-- only if we have a digging tool
								if Waypoint.Type ~= "drop" then
									local PathSegRay = SceneMan:ShortestDistance(PrevWptPos, Waypoint.Pos, false)	-- detect material blocking the path and start digging through it
									if self.teamBlockState ~= Actor.BLOCKED and SceneMan:CastStrengthRay(PrevWptPos, PathSegRay, 5, Free, 1, rte.doorID, true) then
										if SceneMan:ShortestDistance(self.Pos, Free, false).Magnitude < self.Height*0.5 then	-- check that we're close enough to start digging
											-- TODO: update the path and try again just to make sure we can't get through some where else
											digState = AHuman.STARTDIG
											self.deviceState = AHuman.DIGGING
											obstacleState = Actor.DIGPAUSING
											nextLatMove = Actor.LAT_STILL
											sweepRange = math.pi / 4
											StuckTimer:SetSimTimeLimitMS(5000)
											self.Ctrl.AnalogAim = SceneMan:ShortestDistance(self.Pos, Waypoint.Pos, false).Normalized	-- aim in the direction of the next waypoint
										else
											digState = AHuman.NOTDIGGING
											obstacleState = Actor.PROCEEDING
										end
										
										coroutine.yield()	-- wait until next frame
									else
										digState = AHuman.NOTDIGGING
										obstacleState = Actor.PROCEEDING
										StuckTimer:SetSimTimeLimitMS(1500)
									end
								end
								
								if digState == AHuman.NOTDIGGING and self.deviceState ~= AHuman.DIGGING then
									-- if our path isn't blocked enough to dig, but the headroom is too little, start crawling to get through
									local Heading = SceneMan:ShortestDistance(self.Pos, Waypoint.Pos, false):SetMagnitude(self.Height*0.5)
									
									-- don't crawl if it's too steep, climb then instead
									if math.abs(Heading.X) > math.abs(Heading.Y) and self.Head and self.Head:IsAttached() then
										local TopHeadPos = self.Head.Pos - Vector(0, self.Head.Radius*0.7)
										
										-- first check up to the top of the head, and then from there forward
										if SceneMan:CastStrengthRay(self.Pos, TopHeadPos - self.Pos, 5, Free, 4, rte.doorID, true) or
												SceneMan:CastStrengthRay(TopHeadPos, Heading, 5, Free, 4, rte.doorID, true)
										then
											self.crawling = true
										else
											self.crawling = false
										end
										
										coroutine.yield()	-- wait until next frame
									else
										self.crawling = false
									end
								end
							end
						end
						
						if not Waypoint.Type then
							ArrivedTimer:SetSimTimeLimitMS(100)
						elseif Waypoint.Type == "last" then
							ArrivedTimer:SetSimTimeLimitMS(500)
						else	-- air or corner wpt
							ArrivedTimer:SetSimTimeLimitMS(25)
						end
					end
				elseif #WptList > 1 then	-- check if some other waypoint is closer
					local test = math.random(1, #WptList)
					local RandomWpt = WptList[test]
					if RandomWpt then
						Dist = SceneMan:ShortestDistance(self.Pos, RandomWpt.Pos, false)
						local mag = Dist.Magnitude
						if mag < 50 and mag < SceneMan:ShortestDistance(self.Pos, Waypoint.Pos, false).Magnitude/3 then
							-- this waypoint is closer, check LOS
							if -1 == SceneMan:CastObstacleRay(self.Pos, Dist, Vector(), Vector(), self.ID, self.IgnoresWhichTeam, rte.grassID, 4) then
								Waypoint = RandomWpt	-- go here instead
								if WptList[test-1] then
									PrevWptPos = WptList[test-1].Pos
								else
									PrevWptPos = self.Pos
								end
								
								test = math.min(test, #WptList)
								for i = 1, test do	-- delete the earlier waypoints
									table.remove(WptList, 1)
									if #WptList > 0 then
										self:RemoveMovePathBeginning()
									end
								end
							end
						end
					end
				end
				
				if Waypoint then
					CurrDist = SceneMan:ShortestDistance(self.Pos, Waypoint.Pos, false)
					-- not digging
					if not self.Target then
						self.fire = false
					end
					
					-- Scan for obstacles
					local Trace = Vector(self.Diameter*0.85, 0):RadRotate(scanAng)
					local Free = Vector()
					local index = math.floor(scanAng*2.5+2.01)
					if SceneMan:CastObstacleRay(self.Pos, Trace, Vector(), Free, self.ID, self.IgnoresWhichTeam, rte.grassID, 3) > -1 then
						Obstacles[index] = true
					else
						Obstacles[index] = false
					end
					
					if scanAng < 1.57 then	-- pi/2
						if scanAng > 1.2 then
							scanAng = 1.89
						else
							scanAng = scanAng + 0.55
						end
					else
						if scanAng > 3.5 then
							scanAng = -0.4
						else
							scanAng = scanAng + 0.55
						end
					end
					
					coroutine.yield()	-- wait until next frame
					
					if CurrDist.Magnitude > self.Height then	-- not close enough to the waypoint
						ArrivedTimer:Reset()
						
						-- check if we have LOS to the waypoint
						if SceneMan:CastObstacleRay(self.Pos, CurrDist, Vector(), Vector(), self.ID, self.IgnoresWhichTeam, rte.grassID, 9) < 0 then
							NoLOSTimer:Reset()
						elseif NoLOSTimer:IsPastSimTimeLimit() then	-- calulate new path
							Waypoint = nil
							WptList = nil
						end							
						
						if not self.jump and not self.flying then
							coroutine.yield()	-- wait until next frame
						end
					elseif ArrivedTimer:IsPastSimTimeLimit() then	-- only remove a waypoint if we have been close to it for a while
						PrevWptPos = Waypoint.Pos
						self:RemoveMovePathBeginning()
						Waypoint = nil
					end
					
					if Waypoint then	-- move towards the waypoint
						nextAimAngle = self:GetAimAngle(false) * 0.8	-- look straight ahead
	
						if math.abs(CurrDist.X) > 5 and math.abs(CurrDist.Y) > 5 and CurrDist.Y < 20 then
							self.jump = true
						else
							self.jump = false
						end
						
						-- control horisontal movement
						if self.jump then
							change = CurrDist.X+self.Vel.X*3
							
							if change < -0.5 then
								nextLatMove = Actor.LAT_LEFT
							elseif change > 0.5 then
								nextLatMove = Actor.LAT_RIGHT
							else
								nextAimAngle = 1.2	-- look up to aim jetpack down
							end
						else
							if CurrDist.X < -3 then
								nextLatMove = Actor.LAT_LEFT
							elseif CurrDist.X > 3 then
								nextLatMove = Actor.LAT_RIGHT
							else
								nextLatMove = Actor.LAT_STILL
							end
						end
						
						if Waypoint.Type == "right" then
							if CurrDist.X > -3 then
								nextLatMove = Actor.LAT_RIGHT
							end
						elseif Waypoint.Type == "left" then
							if CurrDist.X < 3 then
								nextLatMove = Actor.LAT_LEFT
							end								
						end
						
						-- must fire thrusters to move sideways when in the air... NOT
						--[[
						if not self.jump and self.flying and Waypoint.Type ~= "drop" and
							 self.Vel.Y > -5 and CurrDist.Y-self.Vel.Y*3 < 20	--TODO: use Ychange here?
						then
							change = math.abs(CurrDist.X+self.Vel.X*3)
							if change > 0.5 then
								self.jump = true
							end
						end]]
						
						if self.jump then
							-- obstacle right
							for i = rte.Obst.R_FRONT, rte.Obst.R_UP do
								if Obstacles[i] then
									if Obstacles[rte.Obst.R_UP] then
										self.jump = false
									else
										nextAimAngle = 1.2	-- look up
									end
									
									break
								end
							end
							
							-- obstacle left
							for i = rte.Obst.L_UP, rte.Obst.L_FRONT do
								if Obstacles[i] then
									if Obstacles[rte.Obst.L_UP] then
										self.jump = false
									else
										nextAimAngle = 1.2	-- look up
									end
									
									break
								end
							end
						elseif Waypoint.Type ~= "drop" and not Lower(Waypoint, self, 20) then						
							-- jump over low obstacles unless we want to jump off a ledge
							if nextLatMove == Actor.LAT_RIGHT and
								(Obstacles[rte.Obst.R_LOW] or Obstacles[rte.Obst.R_FRONT]) and not Obstacles[rte.Obst.R_UP]
							then
								self.jump = true
								if Obstacles[rte.Obst.R_HIGH] then
									nextLatMove = Actor.LAT_LEFT -- TODO: only when too close to the obstacle?
								end
							elseif nextLatMove == Actor.LAT_LEFT and
								(Obstacles[rte.Obst.L_LOW] or Obstacles[rte.Obst.L_FRONT]) and not Obstacles[rte.Obst.L_UP]
							then
								self.jump = true
								if Obstacles[rte.Obst.L_HIGH] then
									nextLatMove = Actor.LAT_RIGHT -- TODO: only when too close to the obstacle?
								end
							end
						end
						
						if math.abs(self.RotAngle) > 0.5 then
							self.jump = false
						end
					end
					
					if not self.Target then
						self:SetAimAngle(nextAimAngle)
					end

				end
			end
		else	-- no waypoint list, create one in several small steps to reduce lag
			self:DrawWaypoints(false)
			self:UpdateMovePath()
			coroutine.yield()	-- wait until next frame
			
			if self.MovePathSize < 1 then
				break
			end
			
			-- copy the MovePath to a temporary table so we can yield safely while working on the path
			local PathDump = {}
			for WptPos in self.MovePath do
				table.insert(PathDump, WptPos)
			end
			
			coroutine.yield()	-- wait until next frame
			
			-- copy useful waypoints to a temporary path
			local TmpWpts = {}
			table.insert(TmpWpts, {Pos=self.Pos})
			local Origin
			local LastPos = PathDump[1]
			for _, WptPos in pairs(PathDump) do
				Origin = TmpWpts[#TmpWpts].Pos
				WptPos = SceneMan:MovePointToGround(WptPos, self.Height/4, 4)
				if SceneMan:ShortestDistance(Origin, WptPos, false).Magnitude > 100 or	-- skip any waypoint too close to the previous one
					SceneMan:CastStrengthSumRay(Origin, WptPos, 3, rte.grassID) > 5
				then
					table.insert(TmpWpts, {Pos=LastPos})
				end
				
				LastPos = WptPos
				coroutine.yield()	-- wait until next frame
			end
			
			table.insert(TmpWpts, {Pos=PathDump[#PathDump]})	-- add the last waypoint in the MovePath
			
			coroutine.yield()	-- wait until next frame
			
			WptList = {}	-- store the waypoints we want in our path here
			local StartWpt = table.remove(TmpWpts, 1)
			while #TmpWpts > 0 do
				local NextWpt = table.remove(TmpWpts, 1)
				
				if Lower(NextWpt, StartWpt, 30) then	-- scan for sharp drops	TODO: check the slope instead?
					NextWpt.Type = "drop"
					
					local GapList = {}
					for j, JumpWpt in pairs(TmpWpts) do	-- look for the other side
						local Gap = SceneMan:ShortestDistance(StartWpt.Pos, JumpWpt.Pos, false)
						if Gap.Magnitude > 400 - Gap.Y then	-- TODO: use actor properties here
							break	-- too far
						end
						
						if Gap.Y > -40 then	-- no more than 2m above
							table.insert(GapList, {Wpt=JumpWpt, score=math.abs(Gap.X/Gap.Y), index=j})
						end
					end
					
					coroutine.yield()	-- wait until next frame
					
					table.sort(GapList, function(A, B) return A.score > B.score end)	-- sort largest first
					
					for _, LZ in pairs(GapList) do
						-- check if we can jump
						local Trace = SceneMan:ShortestDistance(StartWpt.Pos, LZ.Wpt.Pos, false)
						if -1 == SceneMan:CastObstacleRay(StartWpt.Pos, Trace, Vector(), Vector(), self.ID, self.IgnoresWhichTeam, rte.grassID, 4) then							
							-- find a point mid-air
							local TestPos = StartWpt.Pos + Trace / 2
							local Free = Vector()
							if 0 ~= SceneMan:CastObstacleRay(TestPos, Vector(0, -math.abs(Trace.X)/2), Vector(), Free, self.ID, self.IgnoresWhichTeam, rte.grassID, 2) then	-- TODO: check LOS? what if 0?
								table.insert(WptList, {Pos=Free+Vector(0,self.Height/4), Type="air"})	-- guide point in the air
								NextWpt = LZ.Wpt
								
								-- delete any waypoints between StartWpt and the LZ
								for i = LZ.index, 1, -1 do
									table.remove(TmpWpts, i)
								end
								
								break
							end
						end
						
						coroutine.yield()	-- wait until next frame
					end
				end
				
				table.insert(WptList, NextWpt)
				StartWpt = NextWpt
			end
			
			WptList[#WptList].Type = "last"
			
			coroutine.yield()	-- wait until next frame
			
			-- create the move path seen on the screen
			self:ClearMovePath()
			for _, Wpt in pairs(WptList) do
				self:AddToMovePathEnd(Wpt.Pos)
			end
			
			self:DrawWaypoints(true)
			NoLOSTimer:Reset()
		end
		
		-- movement commands
		if self.Target and self.jump then	-- don't issue body commands if we have a target in mid-air
			self.lateralMoveState = Actor.LAT_STILL
		else
			self.lateralMoveState = nextLatMove
		end
		
		if self.BlockingActor then
			if not MovableMan:IsActor(self.BlockingActor) or
				SceneMan:ShortestDistance(self.Pos, self.BlockingActor.Pos, false).Magnitude > (self.Diameter + self.BlockingActor.Diameter)*1.2
			then
				self.BlockingActor = nil
				self.teamBlockState = Actor.NOTBLOCKED
				
				if self.AIMode == Actor.AIMODE_BRAINHUNT and self.FollowingActor then
					self.FollowingActor = nil
					break	-- end this behavior
				end
			elseif self.teamBlockState == Actor.NOTBLOCKED and Waypoint then
				if (Waypoint.Pos.X > self.Pos.X and self.BlockingActor.Pos.X > self.Pos.X) or
					 (Waypoint.Pos.X < self.Pos.X and self.BlockingActor.Pos.X < self.Pos.X)
				then
					self.teamBlockState = Actor.BLOCKED
					if self.AIMode == Actor.AIMODE_BRAINHUNT and self.AIMode == self.BlockingActor.AIMode and
						(not self.BlockingActor.MOMoveTarget or self.BlockingActor.MOMoveTarget.ID ~= self.ID)
					then	-- don't follow an actor that is following us
						self.FollowingActor = self.BlockingActor
						self:ClearAIWaypoints()
						self:AddAIMOWaypoint(self.FollowingActor)
						self:CreateGoToBehavior()
					end
				else
					self.BlockingActor = nil
				end
			end
		end
		
		coroutine.yield()	-- wait until next frame
	end
	
	return true
end

-- stop the user fom inadvertently modifying the storage table
local Proxy = {}
local Mt = {
	__index = CrabBehaviors,
	__newindex = function(Table, k, v)
		error("The CrabBehaviors table is read-only.", 2)
	end
}
setmetatable(Proxy, Mt)
CrabBehaviors = Proxy
