dofile("Base.rte/Actors/AI/HumanAI.lua")

function Update(self)
--Just here to see notifications of this running.
--print("running the movement script!")

--Create the necessary variables.
if self.timer == nil then
	self.timer = Timer();
end

--Start our loop, every 10-30ms.  Randomly spaced for (slightly) better load distribution.
local randomTimer = math.random(10,30);
if self.timer:IsPastSimMS(randomTimer) then
	local p = Vector();

	--Core of the Movement script starts here.
	--Important bit here- adjust the altitude value until it's stable and not too floaty.  
	--This specific altitude should be slightly lower than the AHuman's hip center to the ground.
	local altCheck = 19
	local heightCheck = 30
	local sideCheck = 10
	local upPush = 0.35
	local downPush = 0.7
	local downCheck = false
	local upCheck = false
	
	--Air / Ground movement speeds and the limit where Actors can use faster air speeds.
	local airLimit = 150	
	local air_speed_max = 12.0
	local air_accel = 1.25
	local speed_max = 8.0
	local accel = 1.0
	
	--Jumping variables.
	local jump_speed_max = 6.0
	local jump_accel = 2.0

	local centerAlt = SceneMan:CastObstacleRay(self.Pos, Vector(0,altCheck), p, p, self.ID, 0, 0, 0);
	local frontAlt = SceneMan:CastObstacleRay(Vector(self.Pos.X-4,self.Pos.Y), Vector(0,altCheck), p, p, self.ID, 0, 0, 0);
	local backAlt = SceneMan:CastObstacleRay(Vector(self.Pos.X+4,self.Pos.Y), Vector(0,altCheck), p, p, self.ID, 0, 0, 0);
	
	--Safety code, because we get -1 if no obstacles
	if centerAlt < 0 then
		centerAlt = 1000
	end
	if frontAlt < 0 then
		frontAlt = 1000
	end
	if backAlt < 0 then
		backAlt = 1000
	end

	--Now that we know the heights to obstacles, it's time to perform some logic.
	--May need to alter the velPushfor best performance depending on mass.
	--Please note that this allows for ducking and prone and even movement while body is lowered; let go of the control...
	--...and the character "stands" back up again.
	if (centerAlt < altCheck or frontAlt < altCheck or backAlt < altCheck) and not (self:GetController():IsState(Controller.MOVE_DOWN) or self:GetController():IsState(Controller.BODY_CROUCH) or self:GetController():IsState(Controller.HOLD_DOWN)) then
		downCheck = 2
	end
	if (centerAlt < altCheck or frontAlt < altCheck or backAlt < altCheck) and (self:GetController():IsState(Controller.MOVE_DOWN) or self:GetController():IsState(Controller.BODY_CROUCH) or self:GetController():IsState(Controller.HOLD_DOWN)) then
		self.Vel.Y = self.Vel.Y + 0.1
		downCheck = 1
	end
	if not (centerAlt < altCheck or frontAlt < altCheck or backAlt < altCheck) and not (self:GetController():IsState(Controller.MOVE_DOWN) or self:GetController():IsState(Controller.BODY_CROUCH) or self:GetController():IsState(Controller.HOLD_DOWN)) then
		downCheck = 0
	end

	--Do a raytest vs. hitting top of the Actor on things.  If buried, push down, hard.
	local ray = SceneMan:CastObstacleRay(self.Pos, Vector(4,-heightCheck), p, p, self.ID, 0, 0, 0);
	local ray2 = SceneMan:CastObstacleRay(self.Pos, Vector(-4,-heightCheck), p, p, self.ID, 0, 0, 0);
	if (ray < heightCheck and ray ~= -1) or (ray2 < heightCheck and ray2 ~= -1) then
		upCheck = true
	else 
		upCheck = false 
	end

	--If we're supposed to push DOWN or UP, do one or the other, but if both conditions... 
	--...are true or false, do NOTHING.  Prevents odd behaviors underground
	if downCheck == 2 and upCheck == false then
		self.Vel.Y = self.Vel.Y - upPush
	end	
	if downCheck == 0 and upCheck == true then
		self.Vel.Y = self.Vel.Y + downPush
	end

	--Jump Script; alters Y velocity when jumping.
	if self:GetController():IsState(Controller.BODY_JUMPSTART) or self:GetController():IsState(Controller.BODY_JUMP) then	
		if upCheck == false and self.JetTimeLeft > TimerMan.DeltaTimeMS * 3 then
			self.Vel.Y = math.max(self.Vel.Y - jump_accel,-jump_speed_max)
		end
	end

	--Do raytests to the right and left to make sure we're not running through walls.
	--This isn't exactly super-reliable, but it's better'n nothin'.
	local rightLeft = 0
	ray = SceneMan:CastObstacleRay(self.Pos, Vector(sideCheck,0), p, p, self.ID, 0, 0, 0);
	if ray < sideCheck and ray ~= -1 then
		rightLeft = rightLeft -1
	end
	ray = SceneMan:CastObstacleRay(self.Pos, Vector(-sideCheck,0), p, p, self.ID, 0, 0, 0);
	if ray < sideCheck and ray ~= -1 then
		rightLeft = rightLeft + 1
	end
	--If we want to push right OR left, but NOT push right AND left, then push right or left.
	--Sorry for the logic joke.
	if rightLeft == -1 then
		self.Vel.X = self.Vel.X - 1.0
	end
	if rightLeft == 1 then
		self.Vel.X = self.Vel.X + 1.0
	end	

	--Brute-force change in velocity over time. 
	--If accel is set really high, bad things will happen ;-)
	--Now allows for faster movement / accel if in the air.
	local airAlt = SceneMan:CastObstacleRay(self.Pos, Vector(0,airLimit), p, p, self.ID, 0, 0, 0);
	if airAlt == nil then airAlt = -1 end
	if self:GetController():IsState(Controller.MOVE_LEFT) and rightLeft == 0 and downCheck ~= 1 then
		if airAlt == -1 then
			self:AddForce(Vector(-air_accel * 1250,0),Vector(0,0));
		else 	
			self.Vel.X = math.max(self.Vel.X - accel,-speed_max)
		end
	end
	if self:GetController():IsState(Controller.MOVE_RIGHT) and rightLeft == 0 and downCheck ~= 1 then
		if airAlt == -1 then
			self:AddForce(Vector(air_accel * 1250,0),Vector(0,0));
		else 	
			self.Vel.X = math.min(self.Vel.X + accel,speed_max)
		end
	end
	--Use air resistance to stop quickly in mid-air.
	if math.abs(self.Vel.X) > 3.0 and airAlt == -1 then
		if not self:GetController():IsState(Controller.MOVE_RIGHT) and self.Vel.X > 3.0 then
			self.Vel.X = self.Vel.X * 0.95;
		end
		if not self:GetController():IsState(Controller.MOVE_LEFT) and self.Vel.X < -3.0 then
			self.Vel.X = self.Vel.X * 0.95;
		end
	end

	--Fix for absurd rotations while in mid-air.
	if airAlt == -1 then
		if self.RotAngle > 0.1 then
			self.RotAngle = 0.1
		end	
		if self.RotAngle < - 0.1 then
			self.RotAngle = - 0.1
		end
	end

	if self.Controller == nil then
		self.Controller = self:GetController()
	end	
	--Regulate how fast we travel downwards.
	if self.Vel.Y > 8.0 and not self:GetController():IsState(Controller.MOVE_DOWN) then
		self.Controller:SetState(Controller.BODY_JUMP, true)
		self.Vel.Y = self.Vel.Y - 0.5
	end	
	

	--Brute-force removal of "falling through the earth"; troops hit "bedrock".
	if self.Pos.Y > SceneMan.SceneHeight - 40 then 
		self.Pos.Y = SceneMan.SceneHeight - 42
		self.Vel.Y = -1.0
	end		

	--Reset our 10ms timer
	self.timer:Reset();
--End our loop
end
--End the Update() cycle
end